from nmigen import *
from nmigen.sim import Simulator
from clock_mux import ClockMux
from clock_divider import ClockDivider

SRCFREQ = 20e6
OFREQ1 = 10e6
OFREQ2 = 2e6

if __name__ == '__main__':
    m = Module()

    mux_sel = Signal()
    test_sig = Signal()

    m.submodules.c1 = c1 = ClockDivider(SRCFREQ, OFREQ1)
    m.submodules.c2 = c2 = ClockDivider(SRCFREQ, OFREQ2)
    m.submodules.mux = mux = ClockMux()

    m.domains.out = ClockDomain("out")

    m.d.comb += [
        mux.i_sel.eq(mux_sel),
        mux.i_clka.eq(c1.o),
        mux.i_clkb.eq(c2.o),
        ClockSignal("out").eq(mux.o_clk)
    ]

    m.d.out += test_sig.eq(~test_sig)

    sim = Simulator(m)
    sim.add_clock(1/SRCFREQ)

    def test():
        for _ in range(50):
            yield
        yield mux_sel.eq(1)
        for _ in range(50):
            yield
        yield mux_sel.eq(0)
        for _ in range(50):
            yield

    sim.add_sync_process(test)

    with sim.write_vcd("clock_bug.vcd", "clock_bug.gtkw", traces=[mux_sel, c1.o, c2.o, mux.o_clk, test_sig]):
        sim.run()
